package broker_test

import (
	"errors"

	"github.com/pivotal-cf/on-demand-service-broker/broker"
	"github.com/pivotal-cf/on-demand-service-broker/mgmtapi"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("Instances", func() {
	Describe("listing all instances", func() {
		Context("of which there are three", func() {
			JustBeforeEach(func() {
				instances := []broker.Instance{
					{ID: "red", PlanID: "plan-guid"},
					{ID: "green", PlanID: "plan-guid"},
					{ID: "blue", PlanID: "plan-guid"},
				}
				instanceRepo.AllInstancesReturns(instances, nil)
			})

			It("returns a list of instances with ID, pland ID and deployment name", func() {
				red := mgmtapi.Instance{
					InstanceID:         "red",
					PlanID:             "plan-guid",
					BoshDeploymentName: "service-instance_red",
				}
				green := mgmtapi.Instance{
					InstanceID:         "green",
					PlanID:             "plan-guid",
					BoshDeploymentName: "service-instance_green",
				}
				blue := mgmtapi.Instance{
					InstanceID:         "blue",
					PlanID:             "plan-guid",
					BoshDeploymentName: "service-instance_blue",
				}
				Expect(b.Instances()).To(ConsistOf(red, green, blue))
			})
		})

		Context("but failing to do so", func() {
			JustBeforeEach(func() {
				instanceRepo.AllInstancesReturns(nil, errors.New("fnuh"))
			})

			It("returns HTTP 500", func() {
				_, err := b.Instances()
				Expect(err).To(MatchError(ContainSubstring("fnuh")))
			})
		})
	})
})
