package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
)

func (b *Broker) Unbind(instanceID, bindingID string, details brokerapi.UnbindDetails) error {
	errs := func(context string, err error) error {
		wrappedErr := fmt.Errorf("error %s: %s", context, err)
		return convertToDisplayableError(wrappedErr, b.logger)
	}
	if exists, err := b.instanceRepo.InstanceExists(instanceID); !exists {
		return brokerapi.ErrInstanceDoesNotExist
	} else if err != nil {
		return errs("finding instance", err)
	}

	vms, manifest, err := b.gatherBindingContext(instanceID)
	if err != nil {
		return errs("gathering unbinding info", err)
	}

	requestParams := map[string]interface{}{
		"plan_id":    details.PlanID,
		"service_id": details.ServiceID,
	}

	b.logger.Printf("service adapter will delete binding with ID %s for instance %s\n", bindingID, instanceID)
	if err := b.serviceAdapter.DeleteBinding(bindingID, vms, manifest, requestParams); err != nil {
		return errs("deleting binding", err)
	}

	return nil
}
